/*---------------------------------------------------------------------------
* 							Dr. Fritz Faulhaber GmbH & Co.KG
*----------------------------------------------------------------------------
*//**\file
* \brief    Implementation of class TFormMain
* 	Provides the interaction with the demo user interface,
* 	uses CMomanLibSample to get access to the FAULHABER communiction API.
*/
//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "MainVCL.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TFormMain *FormMain;
//---------------------------------------------------------------------------

const int cNodeNr = 1;		//node number of the connected device
//---------------------------------------------------------------------------

//***************
//*** Global: ***
//***************
void CBAsyncDataReceived(void)
{
	//Use windows message to signal asynchronous received data to WMComData()
	PostMessage(FormMain->Handle, WM_COMDATA, 0, 0);
}
//---------------------------------------------------------------------------

//***************
//*** Public: ***
//***************
__fastcall TFormMain::TFormMain(TComponent* Owner)
	: TForm(Owner)
{
	MomanLibSample = new CMomanLibSample();
	if (MomanLibSample->Init(CBAsyncDataReceived) == true) {
		m_bMomanLibLoaded = true;
		StatusBar1->SimpleText = "FAULHABER communication API loaded";
		ButtonGetStrObj->Enabled = true;
		ButtonShutdown->Enabled = true;
		ButtonSwitchOn->Enabled = true;
		ButtonEnOp->Enabled = true;
	}
	else {
		m_bMomanLibLoaded = false;
		ShowMessage("Interface can not be opened!");
	}
}
//---------------------------------------------------------------------------

__fastcall TFormMain::~TFormMain()
{
	delete MomanLibSample;
}
//---------------------------------------------------------------------------


//****************
//*** Private: ***
//****************
void __fastcall TFormMain::WMComData(TMessage& Msg)
{
	//Asynchronous data received
	std::string data, cmd;
	MomanLibSample->ReadReceivedData(data, cmd);
	if (cmd != "") {
		RichEdit1->Lines->Add("Async Received: " + String(data.c_str()) +
									 " (Cmd = " + String(cmd.c_str()) +")");
	}
	else {
		RichEdit1->Lines->Add("Async Received: " + String(data.c_str()));
	}

	if ((MomanLibSample->GetStatusword() & 0x00FF) == 0x0027) {	//"Operation Enabled"
		ButtonRelPos->Enabled = true;
	}
	else {
		ButtonRelPos->Enabled = false;
	}
}
//---------------------------------------------------------------------------


//******************
//*** Published: ***
//******************
void __fastcall TFormMain::ButtonGetStrObjClick(TObject *Sender)
{
	if (m_bMomanLibLoaded) {
		//Read device name
		std::string value;
		MomanLibSample->GetStrObj(cNodeNr, 0x1008, 0x00, value);
		RichEdit1->Lines->Add("Sync Read 0x1008.00");
		RichEdit1->Lines->Add("Sync Received: " + String(value.c_str()));
	}
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::ButtonShutdownClick(TObject *Sender)
{
	if (m_bMomanLibLoaded) {
		if (MomanLibSample->SendCommand(cNodeNr, eMomancmd_shutdown) == true) {
			RichEdit1->Lines->Add("Send SHUTDOWN");
		}
	}
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::ButtonSwitchOnClick(TObject *Sender)
{
	if (m_bMomanLibLoaded) {
		if (MomanLibSample->SendCommand(cNodeNr, eMomancmd_switchon) == true) {
			RichEdit1->Lines->Add("Send SWITCHON");
		}
	}
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::ButtonEnOpClick(TObject *Sender)
{
	if (m_bMomanLibLoaded) {
		if (MomanLibSample->SendCommand(cNodeNr, eMomancmd_EnOp) == true) {
			RichEdit1->Lines->Add("Send ENOP");
		}
	}
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::ButtonRelPosClick(TObject *Sender)
{
bool err = true;

	RichEdit1->Lines->Add("Execute MOVE RELATIVE");
	//Modes of Operation = Profile Position Mode (1):
	if (MomanLibSample->SetObj(cNodeNr, 0x6060, 0x00, 1, 1) == true) {
		//Target Position = 1000:
		if (MomanLibSample->SetObj(cNodeNr, 0x607A, 0x00, 1000, 4) == true) {
         //Enable Operation:
			if (MomanLibSample->SendCommand(cNodeNr, eMomancmd_EnOp) == true) {
				//Move relative:
				if (MomanLibSample->SetObj(cNodeNr, 0x6040, 0x00, 0x007F, 2) == true) {
					err = false;
				}
			}
		}
	}

	if (err) {
		std::string abortMessage = MomanLibSample->GetAbortMessage();
		if (abortMessage != "") {
			RichEdit1->Lines->Add("Error Received: " + String(abortMessage.c_str()));
		}
	}

}
//---------------------------------------------------------------------------

