{*---------------------------------------------------------------------------
* 							Dr. Fritz Faulhaber GmbH & Co.KG
*----------------------------------------------------------------------------
* Main form of the demo example
*----------------------------------------------------------------------------
*}
unit Main;

//---------------------------------------------------------------------------
interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.ComCtrls, Vcl.StdCtrls,
  MomanLibSample;

const
  WM_COMDATA = WM_USER + 1; //windows message to signal asynchronous received data
  cNodeNr = 1;              //node number of the connected device

type
  TFormMain = class(TForm)
    ButtonEnOp: TButton;
    ButtonGetStrObj: TButton;
    ButtonRelPos: TButton;
    ButtonShutdown: TButton;
    ButtonSwitchOn: TButton;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    RichEdit1: TRichEdit;
    StatusBar1: TStatusBar;
    procedure FormCreate(Sender: TObject);
    procedure ButtonGetStrObjClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ButtonShutdownClick(Sender: TObject);
    procedure ButtonSwitchOnClick(Sender: TObject);
    procedure ButtonEnOpClick(Sender: TObject);
    procedure ButtonRelPosClick(Sender: TObject);
  private
    { Private-Deklarationen }
	  m_bMomanLibLoaded: Boolean;

    procedure OnWMComData(var Msg: TMessage); message WM_COMDATA;
  public
    { Public-Deklarationen }
  end;

var
  FormMain: TFormMain;
  MomanLibSample: TMomanLibSample;

//---------------------------------------------------------------------------
implementation

{$R *.dfm}

//***************
//*** Global: ***
//***************
procedure CBAsyncDataReceived; stdcall;
begin
	//Use windows message to signal asynchronous received data to OnWMComData()
  PostMessage(FormMain.Handle, WM_COMDATA, 0, 0);
end;

//****************
//*** Private: ***
//****************
procedure TFormMain.OnWMComData(var Msg: TMessage);
var
  data: String;
  cmd: String;
begin
	//Asynchronous data received
	MomanLibSample.ReadReceivedData(data, cmd);
	if cmd <> '' then
  begin
		RichEdit1.Lines.Add('Async Received: ' + String(data) +
									      ' (Cmd = ' + String(cmd) + ')');
	end
	else
  begin
		RichEdit1.Lines.Add('Async Received: ' + String(data));
  end;

	if (MomanLibSample.GetStatusword() and $00FF) = $0027 then	//"Operation Enabled"
  begin
		ButtonRelPos.Enabled := true;
  end
	else
  begin
		ButtonRelPos.Enabled := false;
  end;
end;

//******************
//*** Published: ***
//******************
procedure TFormMain.FormCreate(Sender: TObject);
begin
  MomanLibSample := TMomanLibSample.Create;
  if MomanLibSample.Init(CBAsyncDataReceived) = true then begin
    self.m_bMomanLibLoaded := true;
    StatusBar1.SimpleText := 'FAULHABER communication API loaded';
    ButtonGetStrObj.Enabled := true;
    ButtonShutdown.Enabled := true;
    ButtonSwitchOn.Enabled := true;
    ButtonEnOp.Enabled := true;
  end
  else begin
    ShowMessage('Interface can not be opened!');
    self.m_bMomanLibLoaded := false;
  end
end;

procedure TFormMain.FormDestroy(Sender: TObject);
begin
  MomanLibSample.Free;
end;

procedure TFormMain.ButtonGetStrObjClick(Sender: TObject);
var
  value: String;
begin
    MomanLibSample.GetStrObj(cNodeNr, $1008, $00, value);
    RichEdit1.Lines.Add('Sync Read 0x1008.00');
    RichEdit1.Lines.Add('Sync Received: ' + String(value));
end;

procedure TFormMain.ButtonShutdownClick(Sender: TObject);
begin
	if MomanLibSample.SendCommand(cNodeNr, eMomancmd_shutdown) = true then
			RichEdit1.Lines.Add('Send SHUTDOWN');
end;

procedure TFormMain.ButtonSwitchOnClick(Sender: TObject);
begin
	if MomanLibSample.SendCommand(cNodeNr, eMomancmd_switchon) = true then
			RichEdit1.Lines.Add('Send SWITCHON');
end;

procedure TFormMain.ButtonEnOpClick(Sender: TObject);
begin
	if MomanLibSample.SendCommand(cNodeNr, eMomancmd_EnOp) = true then
			RichEdit1.Lines.Add('Send ENOP');
end;

procedure TFormMain.ButtonRelPosClick(Sender: TObject);
var
  err: Boolean;
  abortMessage: String;
begin
  err := true;
	RichEdit1.Lines.Add('Execute MOVE RELATIVE');
	//Modes of Operation = Profile Position Mode (1):
	if MomanLibSample.SetObj(cNodeNr, $6060, $00, 1, 1) = true then
  begin
		//Target Position = 1000:
		if MomanLibSample.SetObj(cNodeNr, $607A, $00, 1000, 4) = true then
    begin
      //Enable Operation:
			if MomanLibSample.SendCommand(cNodeNr, eMomancmd_EnOp) = true then
      begin
				//Move relative:
				if MomanLibSample.SetObj(cNodeNr, $6040, $00, $007F, 2) = true then
        begin
					err := false;
				end;
      end;
		end;
	end;

	if (err) then
  begin
		abortMessage := MomanLibSample.GetAbortMessage();
		if abortMessage <> '' then
			RichEdit1.Lines.Add('Error Received: ' + abortMessage);
  end;
end;

//---------------------------------------------------------------------------
end.


